# ------------------------------------ DESCRIPTION ------------------------------------
#      Windows 2008/2012 RDS     .
# -      RDP
# -    
# -     (       )
# -      (        )
# -     
# -   ,    ,  DeleteInactivePortSilently.exe,   
# -      RDP

# ------------------------------------- VARIABLES -------------------------------------
#   
$ProfileFolder = "C:\Users"
#   
$ExcludedProfiles = "Default", "Default User", "Public", "All Users", "Administrator"
#    
$ProfileRegistrySection = "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows NT\CurrentVersion\ProfileList"
#    
$ExcludedRegistrySections = "S-1-5-18", "S-1-5-19", "S-1-5-20"
#     
$DomainGroup = [ADSI]"WinNT://domain.com/RDS_Application_Users,group"
#     
$LocalGroup = [ADSI]"WinNT://./Remote Desktop Users,group"
# -------------------------------------------------------------------------------------

#    
$ScriptFolder = $MyInvocation.MyCommand.Path.SubString(0,($MyInvocation.MyCommand.Path.Length - $MyInvocation.MyCommand.Name.Length))
#    -
$LogFile = $ScriptFolder + $MyInvocation.MyCommand.Name.SubString(0,($MyInvocation.MyCommand.Name.Length - 4)) + ".log"
#  -
Out-File -FilePath $LogFile
#   
$OSName = (Get-WmiObject -class Win32_OperatingSystem).Caption

#       
$ScriptTools = $ScriptFolder + $MyInvocation.MyCommand.Name.SubString(0,($MyInvocation.MyCommand.Name.Length - 4)) + "\"
$ScriptToolsArchive = $ScriptFolder + $MyInvocation.MyCommand.Name.SubString(0,($MyInvocation.MyCommand.Name.Length - 4)) + ".zip"
If (Test-Path $ScriptToolsArchive -PathType Leaf) {
  $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
  $LogMessage += "`t"
  $LogMessage += "Script tools are found in " + $ScriptToolsArchive + " archive.`r`n"
  $LogMessage += get-date -uformat "%d.%m.%Y %H:%M:%S"
  $LogMessage += "`t"
  $LogMessage += "Extracting the archive into " + $ScriptTools +"."
  $LogMessage | Out-File -FilePath $LogFile -Append
  
  New-Item -Path $ScriptTools -ItemType directory -Force
  $Shell = New-Object -com shell.application
  $ArchiveItem = $shell.NameSpace($ScriptToolsArchive)
  ForEach($Item in $ArchiveItem.items()) {
    $Shell.Namespace($ScriptTools).copyhere($Item, 0x14)
  }

  If (Test-Path "C:\Scripts\7-Zip\7za.exe" -PathType Leaf) {
    [String]$CmdLine = "C:\Scripts\7-Zip\7za.exe"
    [Array]$CmdLineArg = 'x', """$ScriptToolsArchive""",  "-o""$ScriptTools""", "-y"
    #  & (ampersand) ,     ,   
    #   https://technet.microsoft.com/en-us/library/ee176880.aspx
    $LogMessage = &$CmdLine $CmdLineArg
    $LogMessage += "`r`n"
    $LogMessage | Out-File -FilePath $LogFile -Append
  } else {
    $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
    $LogMessage += "`t"
    $LogMessage += "C:\Scripts\7-Zip\7za.exe is not found. Script execution can't be continue."
    $LogMessage | Out-File -FilePath $LogFile -Append
    Exit
  }
}  

Function Get-ComputerSessions {
  Param(
    [CmdletBinding()] 
    [Parameter(ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true)]
    [String]$Computer
  )
  Process {
    $Report = quser /server:$Computer | Select-Object -Skip 1 | ForEach-Object {
      $CurrentLine = $_.Trim() -Replace '\s+',' ' -Split '\s'
      $HashProps = @{
        UserName = $CurrentLine[0]
        ComputerName = $Computer
      }
      # If session is disconnected, different fields will be selected
      If ($CurrentLine[2] -eq 'Disc') {
        $HashProps.SessionName = $null
        $HashProps.Id = $CurrentLine[1]
        $HashProps.State = $CurrentLine[2]
        $HashProps.IdleTime = $CurrentLine[3]
        $HashProps.LogonTime = $CurrentLine[4..6] -join ' '
      } Else {
        $HashProps.SessionName = $CurrentLine[1]
        $HashProps.Id = $CurrentLine[2]
        $HashProps.State = $CurrentLine[3]
        $HashProps.IdleTime = $CurrentLine[4]
        $HashProps.LogonTime = $CurrentLine[5..7] -join ' '
      }
      New-Object -TypeName PSCustomObject -Property $HashProps |
        Select-Object -Property UserName,ComputerName,SessionName,Id,State,IdleTime,LogonTime
    }
    Return $Report 
  }
}

Function Process-DeleteProfileFolders {
  Param(
    [CmdletBinding()] 
    [Parameter(ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true,Mandatory=$true)]
    [String]$ProfileFolder,
    [Parameter(ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true,Mandatory=$true)]
    [String[]]$ExcludedProfiles,
    [Parameter(ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true,Mandatory=$false)]
    [String]$LogFile
  )
  Process {
    $Report = $Nothing
    #     
    $SubDirs= Get-ChildItem $ProfileFolder -Force
    #     
    ForEach ($Dir in $SubDirs) {
      $LogMessage = $Nothing
      # ,      
      If (Test-Path $Dir.FullName -PathType Container) {
        # ,   -    
        $NotDeleteFlag = $False
        ForEach ($ExcludedProfile in $ExcludedProfiles) {
          If ($Dir.Name -eq $ExcludedProfile) {
            $NotDeleteFlag = $True
          }
        }
        #  
        If ($NotDeleteFlag -eq $False) {
          #    
          $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
          $LogMessage += "`t"
          $LogMessage += $Dir.FullName + " is deleted."
          #  
          $CmdLine = "CMD /c RD /S /Q """ + $Dir.FullName + """"
          Invoke-Expression -command $CmdLine
        } Else {
          #    
          $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
          $LogMessage += "`t"
          $LogMessage += $Dir.FullName + " is skipped as a service folder."
        }
      } Else {
        #    
        $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
        $LogMessage += "`t"
        $LogMessage += $Dir.FullName + " is skipped as a file."
      }
      If ($LogFile -ne $Nothing) {
        $LogMessage | Out-File -FilePath $LogFile -Append
      } else {
        Write-Output $LogMessage
      }
    }
  }
}

Function Process-DeleteProfileRegistry {
  Param(
    [CmdletBinding()] 
    [Parameter(ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true,Mandatory=$true)]
    [String]$ProfileRegistrySection,
    [Parameter(ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true,Mandatory=$true)]
    [String[]]$ExcludedRegistrySections,
    [Parameter(ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true,Mandatory=$false)]
    [String]$LogFile
  )
  Process {
    $Report = $Nothing
    #     Powershell 
    $ProfileRegistrySection = $ProfileRegistrySection.Replace("HKEY_LOCAL_MACHINE","HKLM:")
    #      
    $Sections= Get-ChildItem $ProfileRegistrySection | Select Name
    ForEach ($Section in $Sections) {
      $LogMessage = $Nothing
      # ,   -    
      $NotDeleteFlag = $False
      ForEach ($ExcludedRegistrySection in $ExcludedRegistrySections) {
        If ($Section.Name.Contains($ExcludedRegistrySection)) {
          $NotDeleteFlag = $True
        }
      }
      #    
      If ($NotDeleteFlag -eq $False) {
        #    
        $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
        $LogMessage += "`t"
        $LogMessage += $Section.Name + " is deleted."
        #  
        Remove-Item -Path $Section -Force -Recurse
      } Else {
        #    
        $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
        $LogMessage += "`t"
        $LogMessage += $Section.Name + " is skipped as a service profile."
      }
      If ($LogFile -ne $Nothing) {
        $LogMessage | Out-File -FilePath $LogFile -Append
      } Else {
        Write-Output $LogMessage
      }
    }
  }
}

Function Process-ResetPrintSpooler {
  Param(
    [CmdletBinding()] 
    [Parameter(ValueFromPipeline=$true,ValueFromPipelineByPropertyName=$true,Mandatory=$false)]
    [String]$LogFile
  )
  Process {
    $LogMessage = $Nothing
    $Error.Clear()
    #  
    Try {
      Stop-Service -Name "Spooler" -Force
      $LogMessage += (get-date -uformat "%d.%m.%Y %H:%M:%S") + "`t" + "Print Spooler service is stopped." + "`r`n"
    } Catch {
      $LogMessage += (get-date -uformat "%d.%m.%Y %H:%M:%S") + "`t" + $Error + "`r`n"
    }
    #  
    Try {
        Remove-Item -Path "C:\Windows\System32\spool\PRINTERS\*.*" -Force
        $LogMessage += (get-date -uformat "%d.%m.%Y %H:%M:%S") + "`t" + "Print queue is purged." + "`r`n"
    }
    Catch {
        $LogMessage += (get-date -uformat "%d.%m.%Y %H:%M:%S") + "`t" + $Error + "`r`n"
    }
    #      
    $CmdLine = $ScriptTools + "DeleteInactivePortSilently.exe"
    $LogMessage += (get-date -uformat "%d.%m.%Y %H:%M:%S") + "`t" + "Delete Inactive TS Ports by DeleteInactivePortSilently.exe command.`r`n" + "`r`n"
    $LogMessage += Invoke-Expression -command $CmdLine
    $LogMessage += "`r`n" + "`r`n"
    #  
    Try {
      Start-Service -Name "Spooler"
      $LogMessage += (get-date -uformat "%d.%m.%Y %H:%M:%S") + "`t" + "Print Spooler service is started."
    } Catch {
      $LogMessage += (get-date -uformat "%d.%m.%Y %H:%M:%S") + "`t" + $Error
    }
    #    -
    If ($LogFile -ne $Nothing) {
      $LogMessage | Out-File -FilePath $LogFile -Append
    } Else {
      Write-Output $LogMessage
    }
  }
}

#     -         
$LocalGroup.Remove($DomainGroup.Path)
$LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
$LogMessage += "`t"
$LogMessage += "The server is closed for users. " + $DomainGroup.name + " is removed from " + $LocalGroup.Name + "."
$LogMessage | Out-File -FilePath $LogFile -Append

#    
$Sessions = Get-ComputerSessions -Computer localhost
ForEach ($Session in $Sessions) {
  logoff $Session.Id /server:localhost
  $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
  $LogMessage += "`t"
  $LogMessage += $Session.UserName + " (" + $Session.Id + ") forced to logoff from the server."
  $LogMessage | Out-File -FilePath $LogFile -Append 
}

#    - 30 
$LocalGroup.Add($DomainGroup.Path)
$LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
$LogMessage += "`t"
$LogMessage += "Waiting 30 seconds for users logoff."
$LogMessage | Out-File -FilePath $LogFile -Append
Start-Sleep -s 30

#     
$Sessions = Get-ComputerSessions -Computer localhost
If ($Sessions.Count -gt 0) {
  $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
  $LogMessage += "`t"
  $LogMessage += "Currently logged on users number is " + $Sessions.Count + "."
  $LogMessage | Out-File -FilePath $LogFile -Append

  $Sessions | FT ID, UserName, State, IdleTime, LogonTime | Out-File -FilePath $LogFile -Append

  $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
  $LogMessage += "`t"
  $LogMessage += "The script processing is terminated, please reboot the server to release sessions."
  $LogMessage | Out-File -FilePath $LogFile -Append
} Else {

  $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
  $LogMessage += "`t"
  $LogMessage += "Currently logged on users number is 0."
  $LogMessage | Out-File -FilePath $LogFile -Append
  
  #     (  Windows 2008)
  If ($OSName.Contains("2008")) {
    Process-DeleteProfileFolders -ProfileFolder $ProfileFolder -ExcludedProfiles $ExcludedProfiles -LogFile $LogFile
  } Else {
    $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
    $LogMessage += "`t"
    $LogMessage += "The server operating system is not match for cleaning profile folders."
    $LogMessage | Out-File -FilePath $LogFile -Append
  }

  #     
  Process-DeleteProfileRegistry -ProfileRegistrySection $ProfileRegistrySection -ExcludedRegistrySections $ExcludedRegistrySections -LogFile $LogFile

  #          
  $LocalGroup.Add($DomainGroup.Path)
  $LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
  $LogMessage += "`t"
  $LogMessage += "Delete ""Optimize Start Menu Cache Files"" scheduled tasks."
  $LogMessage | Out-File -FilePath $LogFile -Append
  Get-ScheduledTask | where {$_.taskname -like "Optimize Start Menu Cache Files*"} | Unregister-ScheduledTask -Confirm:$false

  #   
  Process-ResetPrintSpooler -LogFile $LogFile

}

#     -         
$LocalGroup.Add($DomainGroup.Path)
$LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
$LogMessage += "`t"
$LogMessage += "The server is opened for users. " + $DomainGroup.name + " is added to " + $LocalGroup.Name + "."
$LogMessage | Out-File -FilePath $LogFile -Append

#   
$LogMessage = get-date -uformat "%d.%m.%Y %H:%M:%S"
$LogMessage += "`t"
$LogMessage += "$ScriptTools auxiliary folder is deteted."
$CmdLine = "CMD /c RD /S /Q """ + $ScriptTools + """"
Invoke-Expression -command $CmdLine